searchNodes=[{"doc":"Utility functions for dealing with Plug.Conn structs.","ref":"Liberator.Conn.html","title":"Liberator.Conn","type":"module"},{"doc":"Reads the body from the conn, and puts it in the assigns under the key :raw_body.Accepts the same headers as Plug.Conn.read_body/2::length - sets the maximum number of bytes to read from the body for each chunk, defaults to 64_000 bytes:read_length - sets the amount of bytes to read at one time from the underlying socket to fill the chunk, defaults to 64_000 bytes:read_timeout - sets the timeout for each socket read, defaults to 5_000 milliseconds","ref":"Liberator.Conn.html#read_body/2","title":"Liberator.Conn.read_body/2","type":"function"},{"doc":"A behaviour module for compression codecs.Liberator uses this behaviour to help make sure at compile-time that codecs will be called successfully. Include it in your own module for the same peace of mind.","ref":"Liberator.Encoding.html","title":"Liberator.Encoding","type":"behaviour"},{"doc":"Encode a binary into an encoded form, and raises if there's an error.","ref":"Liberator.Encoding.html#c:encode!/1","title":"Liberator.Encoding.encode!/1","type":"callback"},{"doc":"A behaviour module for media type codecs.Liberator uses this behaviour to help make sure at compile-time that codecs will be called successfully. Include it in your own module for the same peace of mind.","ref":"Liberator.MediaType.html","title":"Liberator.MediaType","type":"behaviour"},{"doc":"Encode an Elixir term into an encoded binary, and raise if there's an error.","ref":"Liberator.MediaType.html#c:encode!/1","title":"Liberator.MediaType.encode!/1","type":"callback"},{"doc":"A controller module that understands and respects the HTTP spec.This module implements a Plug handler that allows an endpoint to comply to the HTTP specification, and to do so by just answering a few questions.Define a simple resource like this:defmodule MyFirstResource do use Liberator.Resource def handle_ok(_), do: &quot;Hello world!&quot; endTo add this plug to a Phoenix application, use the Phoenix.Router.forward/4 keyword in your router:scope &quot;/&quot;, MyApp do pipe_through [:browser] forward &quot;/api/resource&quot;, MyFirstResource endIf you're using another Plug-based framework, use Plug.forward/4 once you've matched on the path:defmodule Router do def init(opts), do: opts def call(conn, opts) do case conn do %{host: &quot;localhost&quot;, path_info: [&quot;resources&quot; | rest]} -&gt; Plug.forward(conn, rest, MyFirstResource, opts) _ -&gt; MainRouter.call(conn, opts) end end endThere are lots of decisions to be made during content negotiation, and Liberator lets gives you access to every single one, but it's also built with sensible defaults that let you quickly build up a controller.Content NegotiationThese functions return lists of available values during content negotiation. If the client has an accept header that does not match a value from these lists, the plug will return a 406 Not Acceptable response, and call handle_not_acceptable/1.FunctionDefaultallowed_methods/1[&quot;GET&quot;, &quot;HEAD&quot;]known_methods/1[&quot;GET&quot;, &quot;HEAD&quot;, &quot;OPTIONS&quot;, &quot;PUT&quot;, &quot;POST&quot;, &quot;DELETE&quot;, &quot;PATCH&quot;, &quot;TRACE&quot;]available_media_types/1[&quot;text/plain&quot;, &quot;application/json&quot;]available_languages/1[&quot;*&quot;]available_charsets/1[&quot;UTF-8&quot;]available_encodings/1[&quot;gzip&quot;, &quot;deflate&quot;,&quot;identity&quot;]maximum_entity_length/164_000Liberator supports a few basic defaults to help you get up and running. It uses Jason for application/json media responses, but you can override that, along with the compression handlers. Just include your custom codecs in config.exs under the :liberator key. Media type codecs go under :media_types, and compression goes under :encodings. Here's the default as an example:config :liberator, media_types: %{ &quot;text/plain&quot; =&gt; Liberator.MediaType.TextPlain, &quot;application/json&quot; =&gt; Jason }, encodings: %{ &quot;identity&quot; =&gt; Liberator.Encoding.Identity, &quot;deflate&quot; =&gt; Liberator.Encoding.Deflate, &quot;gzip&quot; =&gt; Liberator.Encoding.Gzip }As long as your codec module implements an encode!/1 function that accepts and returns a response body, Liberator will call it at the right place in the pipeline. Implement the Liberator.MediaType or Liberator.Encoding behaviour for some compile-time assurance that you've implemented the correct function.defmodule MyXmlCodec do @behaviour Liberator.MediaType @impl true def encode!(body) do # your cool new functionality here end endPreconditionsThese functions decide the state of preconditon decisions for the request. Depending on the specific method and request headers, the plug may return a 412 Precondition Failed response, a 304 Not Modified response, or may allow another kind of request to continue.FunctionDefaultlast_modified/1DateTime.utc_now()etag/1nilActionsActions make the necessary changes to the requested entity. You can return either a Plug.Conn struct, or a map, from these functions. However, unlike the decision functions, a nil or false return value does nothing different.FunctionDescriptioninitialize/1Performs any custom initialization you need before the decision tree startsdelete!/1Called for DELETE requestspatch!/1Called for PATCH requestspost!/1Called for POST requestsput!/1Called for PUT requestsHandlersHandlers are called at the very end of the decision tree, and allow you to return content for rendering to the client.FunctionStatushandle_ok/1200handle_options/1200handle_created/1201handle_accepted/1202handle_no_content/1204handle_multiple_representations/1300handle_moved_permanently/1301handle_see_other/1303handle_not_modified/1304handle_moved_temporarily/1307handle_malformed/1400handle_unauthorized/1401handle_forbidden/1403handle_not_found/1404handle_method_not_allowed/1405handle_not_acceptable/1406handle_conflict/1409handle_gone/1410handle_precondition_failed/1412handle_request_entity_too_large/1413handle_uri_too_long/1414handle_unsupported_media_type/1415handle_unprocessable_entity/1422handle_too_many_requests/1429handle_unavailable_for_legal_reasons/1451handle_unknown_method/1501handle_not_implemented/1501handle_service_unavailable/1503DecisionsLiberator supports a whole lot of decisions points. Some of them are needed for next to every resource definition. Others are seldom used or there is no other sensible implementation.Decision callbacks must return a truthy value, which they can optionally wrap in an {:ok, result} tuple. Returning {:error, result} will always invoke the handle_error; see below.If the result of the decision is a map, Liberator will merge that map with the conn's :assigns map. Use this feature to cache data and do work when it makes sense. For example, the exists?/1 callback is a great place to fetch your resource, and you can return it as a map for your later functions to act upon. That would look something like this:def exists?(conn) do id = List.last(conn.path_info) try do Hello.Blog.get_post!(id) rescue Ecto.NoResultsError -&gt; false ArgumentError -&gt; false else post -&gt; %{post: post} end endHere are all the decision functions you can override:FunctionDescriptionDefaultallowed?/1Is the user allowed to make this request?trueauthorized?/1Is necessary authentication information present?truecharset_available?/1Are any of the requested charsets available? Should assign the :charset variable.Uses values at available_charsets/1can_post_to_gone?/1Should we process a POST to a resource that previously existed?falsecan_post_to_missing?/1Should we process a POST to a resource that does not exist?truecan_put_to_missing?/1Should we process a PUT to a resource that does not exist?trueconflict?/1Does the PUT or POST request result in a conflict?falsedelete_enacted?/1Was the delete request finally processed?trueencoding_available?/1Is the requested encoding available? Should assign the :encoding variable.Uses values at available_encodings/1etag_matches_for_if_match?/1Does the etag of the current resource match the If-Match header?Uses value generated by etag/1etag_matches_for_if_none?/1Does the etag of the current resource match the If-None-Match header?Uses value generated by etag/1existed?/1Did the resource exist before?falseexists?/1Does the resource exist?trueknown_content_type?/1Is the Content-Type of the body known?trueknown_method?/1Is the request method known?Uses values at known_methods/1language_available?/1Is the requested language available? Should assign the :language variable.Uses values at available_languages/1media_type_available?/1Is the requested media type available? Should assign the :media_type variale.Uses values at available_media_types/1method_allowed?/1Is the request method allowed for this resource?Uses values at allowed_methods/1modified_since?/1Was the resource modified since the date given in the If-Modified-Since header?Uses value generated by last_modified/1moved_permanently?/1Was the resource moved permanently?falsemoved_temporarily?/1Was the resource moved temporarily?falsemultiple_representations?/1Are there multiple representations for this resource?falsenew?/1Was the resource created by this request?truepayment_required?/1Is payment required before this request can be processed?falsepost_enacted?/1Was the POST request finally processed?trueput_enacted?/1Was the PUT request finally processed?truepatch_enacted?/1Was the PATCH request finally processed?truepost_redirect?/1Should the response redirect after a POST?falseput_to_different_url?/1Should the PUT request be made to a different URL?falseprocessable?/1Is the request body processable?truetoo_many_requests?/1Has the client or user issued too many requests in a period of time?falseservice_available?/1Is the service available?trueunavailable_for_legal_reasons?/1Is the resource not available, for legal reasons?falseuri_too_long?/1Is the request URI too long?falsevalid_content_header?/1Is the Content-Type of the body valid?truevalid_entity_length?/1Is the length of the body valid?Uses value at maximum_entity_length/1well_formed?/1Is the request parseable?trueHandling ErrorsThere is a special handler, named handle_error/3, that is called when any decision, action, or handler function raises an error or returns {:error, result}. It functions much like an action_fallback module does in Phoenix.The handle_error handler is called with the conn, the error that was raised, and the name of the decision, action, or handler that failed. Liberator expects this handler to call Plug.Conn.send_resp/1 or Plug.Conn.send_resp/3, unlike other handlers. This allows you to set the status and body yourself, or even use a Phoenix fallback controller.The default implementation of handle_error works something like this:@impl true def handle_error(conn, _error, _failed_step) do send_resp(conn, 500, &quot;Internal Server Error&quot;) endDebuggingFor every request, Liberator builds a list of the decisions called and their answers. You can access this list with the :trace option of your use statement.Set it to :log for a log message of the full trace.defmodule MyFirstResource do use Liberator.Resource, trace: :log def handle_ok(_), do: &quot;Hello world!&quot; endYou'll get a log message like this:14:57:04.861 [debug] Liberator trace for request &quot;my-very-specific-request-id&quot; to /: 1. initialize: nil (took 1 µs) 2. service_available?: true (took 1 µs) 3. known_method?: true (took 1 µs) 4. uri_too_long?: false (took 1 µs) 5. method_allowed?: true (took 1 µs) 6. well_formed?: true (took 1 µs) 7. authorized?: true (took 1 µs) 8. allowed?: true (took 1 µs) 9. too_many_requests?: false (took 1 µs) 10. payment_required?: false (took 1 µs) 11. valid_content_header?: true (took 1 µs) 12. known_content_type?: true (took 1 µs) 13. valid_entity_length?: true (took 1 µs) 14. is_options?: false (took 1 µs) 15. accept_exists?: false (took 1 µs) 16. accept_language_exists?: false (took 1 µs) 17. accept_charset_exists?: false (took 1 µs) 18. accept_encoding_exists?: false (took 1 µs) 19. processable?: true (took 1 µs) 20. unavailable_for_legal_reasons?: false (took 1 µs) 21. exists?: true (took 1 µs) 22. if_match_exists?: false (took 1 µs) 23. if_unmodified_since_exists?: false (took 1 µs) 24. if_none_match_exists?: false (took 1 µs) 25. if_modified_since_exists?: false (took 1 µs) 26. method_delete?: false (took 1 µs) 27. method_patch?: false (took 1 µs) 28. post_to_existing?: false (took 1 µs) 29. put_to_existing?: false (took 1 µs) 30. multiple_representations?: false (took 1 µs) 31. handle_ok: nil (took 3 µs)Liberator will include the request ID set by the Plug.RequestId plug, if you have it as part of your pipeline.Set the :trace option to :headers so you can get the trace as HTTP headers.defmodule MyFirstResource do use Liberator.Resource, trace: :headers def handle_ok(_), do: &quot;Hello world!&quot; endThis will add a header called x-liberator-trace that will show you the entire set of decisions, in the order they were made.Lastly, Liberator is instrumented with the Telemetry library, and emits events upon the completion of every request. A Resource will emit the following events:[:liberator, :request, :start][:liberator, :request, :stop][:liberator, :request, :exception]All Telemetry events will contain the request ID and request path as metadata. The measurements for :start will contain a key called system_time which is derived by calling :erlang.system_time() The measurements for :stop and :exception will both contain a key called :duration, which is the duration of the request in native time units. The metadata for the :stop event will also contain the execution trace, which is a list of maps with the following keys::step: the name of the function that was executed, or the atoms :start or :stop:result: the value the function returned:timestamp: the time the function was called, as an Elixir date struct:duration: how long the call took, in native time unitsInternationalization and Localization (i18n and l10n)During content negotiation (specifically, in the call to language_available?/1) the accept-language header is inspected, and the :language key is added to conn.assigns. This callback also sets the Gettext locale for the current process to the value it finds, using Gettext.put_locale/1. So, in your handler functions, all you need to do is make your usual gettext calls. Everything else is handled for you.defmodule MyInternationalizedResource do use Liberator.Resource def available_languages(_): [&quot;en&quot;, &quot;es&quot;, &quot;de&quot;, &quot;fr&quot;] def handle_ok(_), do: gettext(&quot;Hello world!&quot;) endAll of the default handler messages are internationalized, but we need help with translations! If you can help with this, please submit a pull request on Liberator's GitHub page!Advanced OverridesLiberator tries to give you access to as much of the program as possible. Lots of the guts are open for you to play around in. You probably won't ever need to mess with this stuff, but it's there if you need it.Overriding DecisionsThese decision points are used internally by Liberator and provide reasonable defaults. Overriding is possible, but not useful in general.FunctionDescriptionaccept_charset_exists?/1Checks if header Accept-Charset exists.accept_encoding_exists?/1Checks if header Accept-Encoding exists.accept_exists?/1Checks if header Accept exists.accept_language_exists?/1Checks if header Accept-Language exists.body_exists?/1Checks if the request has a body.if_match_exists?/1Checks if header If-Match exists.if_match_star?/1Checks if header If-Match is *.if_match_star_exists_for_missing?/1Checks if header If-Match exists for a resource that does not exist.if_modified_since_exists?/1Checks if header If-Modified-Since exists.if_modified_since_valid_date?/1Checks if header If-Modified-Since is a valid HTTP date.if_none_match?/1Checks if the request method to handle failed If-None-Matchif_none_match_exists?/1Checks if header If-None-Match exists.if_none_match_star?/1Checks if header If-None-Match is *.if_unmodified_since_exists?/1Checks if header If-Unmodified-Since exists.if_unmodified_since_valid_date?/1Checks if header If-Unmodified-Since is a valid HTTP date.is_options?/1Checks if the request method is OPTIONSmethod_delete?/1Checks if the request method is DELETEmethod_put?/1Checks if the request method is PUTmethod_patch?/1Checks if the request method is PATCHpost_to_gone?/1Checks if the request method is POST for resources that do not exist anymore.post_to_existing?/1Checks if the request method is POST for resources that do exist.post_to_missing?/1Checks if the request method is POST for resources that do not exist.put_to_existing?/1Checks if the request method is PUT for a resource that exists.Adding DecisionsSince version 1.3, you can even override the decision, handler, and action trees. To override the decision tree, add an option named :decision_tree_overrides into your use statement. The decision tree is a map of atom -&gt; {atom, atom}, where all three atoms should be function names in the module that called use. The first element of the tuple is the next function to call if the key function returns true, and the second element of the tuple is the function to call if the function returns false. Your argument to :decision_tree_overrides will be merged into the default decision tree. For example, here's me overriding the first chunk of the decision tree so that the decision uri_too_long?/1 is completely skipped. That decisions happens right after known_method?/1, so just update that key to call the next decision instead, which is method_allowed?/1defmodule LongUrisResource do use Liberator.Resource, decision_tree_overrides: %{ # instead of known_method?: {:uri_too_long?, :handle_unknown_method} known_method?: {:method_allowed?, :handle_unknown_method} } endEvery function in the decision matrix needs an entry. If you're adding a new decision function of your own, that new decision needs to be in both a result tuple and a key. Otherwise, Liberator will throw an exception. Also note that Liberator cannot detect a cycle in your callbacks, so be careful!Adding HandlersTo override the handler status, or add your own, add an option named :handler_status_overrides to your use statement, with a map of atom -&gt; integer. The integers are the status codes that Liberator will set before calling the actual handler function.If you are adding a new status code to Liberator, you'll also need to set :decision_tree_overrides in order to actually call this new handler, as well as a functions of those names defined in the module that called use. Here's an example of adding a handler for a new status code:defmodule ResourceLikesToParty do use Liberator.Resource, decision_tree_overrides: %{ allowed?: {:likes_to_party?, :handle_forbidden} likes_to_party?: {:handle_likes_to_party, :too_many_requests?} }, handler_status_overrides: %{ handle_likes_to_party: 420 } } def likes_to_party?(_conn), do: Enum.random([true, false]) def handle_likes_to_party(_conn), do: &quot;Hey come party with me sometime.&quot; endIn this example, the likes_to_party?/1 callback is added, and if that function returns false, it will continue on with the pipeline, but if it returns true, then it will call the new handle_likes_to_party/1 callback, and set the status code to 420.Adding ActionsFinally, you can override actions as well. The option is called :action_followup_overrides, and is a map of atom -&gt; atom, where both atoms are functions defined in the module that called use. The first atom is the name of the handler function, like post!/1 or delete!/1. The second atom is the function that will be called immediately after the action.Say you're implementing a WebDAV server using Liberator, and you want to add your own COPY decision, action, and handler. By overriding some internals, this is how you'd do it, and still have the power of the decision tree on your side!defmodule WebDavResource do use Liberator.Resource, decision_tree_overrides: %{ method_delete?: {:delete!, :method_copy?} method_copy?: {:lock_token_valid?, :method_patch?} lock_token_valid?: {:copy!, :handle_locked} }, handler_status_overrides: %{ handle_locked: 423 }, action_followup_overrides: %{ copy!: :respond_with_entity? } } @impl true def available_media_types(_), do: [&quot;application/xml&quot;] @impl true def allowed_methods(_), do: [&quot;COPY&quot;] @impl true def known_methods(_), do: [&quot;GET&quot;, &quot;HEAD&quot;, &quot;OPTIONS&quot;, &quot;PUT&quot;, &quot;POST&quot;, &quot;DELETE&quot;, &quot;PATCH&quot;, &quot;TRACE&quot;, &quot;COPY&quot;] def lock_token_valid?(conn), do: MyWebDavBackend.lock_token_valid?(conn) def copy!(conn), do: MyWebDavBackend.copy(conn) def handle_locked(_conn), do: &quot;Resource Locked&quot; end","ref":"Liberator.Resource.html","title":"Liberator.Resource","type":"behaviour"},{"doc":"Check if the Accept-Charset header exists.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:accept_charset_exists?/1","title":"Liberator.Resource.accept_charset_exists?/1","type":"callback"},{"doc":"Check if the Accept-Encoding header exists.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:accept_encoding_exists?/1","title":"Liberator.Resource.accept_encoding_exists?/1","type":"callback"},{"doc":"Check if the Accept header exists.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:accept_exists?/1","title":"Liberator.Resource.accept_exists?/1","type":"callback"},{"doc":"Check if the Accept-Language header exists.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:accept_language_exists?/1","title":"Liberator.Resource.accept_language_exists?/1","type":"callback"},{"doc":"Check the authentication information in the request to see if it has the necessary permissions.Note the difference between authorized?/1 and allowed?/1. This function checks if the given request is allowed to perform an action, but isn't responsible for checking the presence of authentication information in the first place.By default, always returns true.","ref":"Liberator.Resource.html#c:allowed?/1","title":"Liberator.Resource.allowed?/1","type":"callback"},{"doc":"Returns a list of HTTP methods that this module serves.The methods returned by this function should be upper-case strings, like &quot;GET&quot;, &quot;POST&quot;, etc.","ref":"Liberator.Resource.html#c:allowed_methods/1","title":"Liberator.Resource.allowed_methods/1","type":"callback"},{"doc":"Check for presence ofauthentication information in the request.Note the difference between authorized?/1 and allowed?/1. This function should just check for the presence of authentication information, not the content of it.If you implement this function to return false, your response in handle_unauthorized must include a WWW-Authenticate header field containing a challenge applicable to the requested resource.By default, always returns true.","ref":"Liberator.Resource.html#c:authorized?/1","title":"Liberator.Resource.authorized?/1","type":"callback"},{"doc":"Returns a list of available content charsets.By default, only UTF-8 is supported.","ref":"Liberator.Resource.html#c:available_charsets/1","title":"Liberator.Resource.available_charsets/1","type":"callback"},{"doc":"Returns a list of available response content encodings (AKA compressions).By default, only identity (no compression) is supported.","ref":"Liberator.Resource.html#c:available_encodings/1","title":"Liberator.Resource.available_encodings/1","type":"callback"},{"doc":"Returns a list of available languages.","ref":"Liberator.Resource.html#c:available_languages/1","title":"Liberator.Resource.available_languages/1","type":"callback"},{"doc":"Returns a list of content types that this module serves.The types returned by this function should be valid MIME types, like text/plain, application/json, etc.","ref":"Liberator.Resource.html#c:available_media_types/1","title":"Liberator.Resource.available_media_types/1","type":"callback"},{"doc":"Check if the request has a body.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:body_exists?/1","title":"Liberator.Resource.body_exists?/1","type":"callback"},{"doc":"Callback implementation for Plug.call/2.","ref":"Liberator.Resource.html#call/2","title":"Liberator.Resource.call/2","type":"function"},{"doc":"Decide if we can process a POST to a resource that existed before, or return a 410 Gone response.By default, always returns false.","ref":"Liberator.Resource.html#c:can_post_to_gone?/1","title":"Liberator.Resource.can_post_to_gone?/1","type":"callback"},{"doc":"Check if we can process a post to a resource that does not exist, or if we should send a 404 Not Found response.By default, always returns true.","ref":"Liberator.Resource.html#c:can_post_to_missing?/1","title":"Liberator.Resource.can_post_to_missing?/1","type":"callback"},{"doc":"Decide if we can PUT to a resource that does not exist, or return 501 Not Implemented.By default, always returns true.","ref":"Liberator.Resource.html#c:can_put_to_missing?/1","title":"Liberator.Resource.can_put_to_missing?/1","type":"callback"},{"doc":"Check of the requested charset is available.By default, uses the values returned by available_charsets/1, and returns a map with the key :charset set to the negotiated charset, which will be merged into conn.assigns.","ref":"Liberator.Resource.html#c:charset_available?/1","title":"Liberator.Resource.charset_available?/1","type":"callback"},{"doc":"Does the PUT or POST request result in a conflict?","ref":"Liberator.Resource.html#c:conflict?/1","title":"Liberator.Resource.conflict?/1","type":"callback"},{"doc":"Called for DELETE requests.","ref":"Liberator.Resource.html#c:delete!/1","title":"Liberator.Resource.delete!/1","type":"callback"},{"doc":"Check if the DELETE request was processed. Return false here if the request was put on some processing queue and the delete was not actually enacted yet. Returning false here would return a 202 Accepted instead of some other response.By default, always returns true.","ref":"Liberator.Resource.html#c:delete_enacted?/1","title":"Liberator.Resource.delete_enacted?/1","type":"callback"},{"doc":"Check of the requested encoding is available.By default, uses the values returned by available_encodings/1, and returns a map with the key :encoding set to the negotiated encoding, which will be merged into conn.assigns.","ref":"Liberator.Resource.html#c:encoding_available?/1","title":"Liberator.Resource.encoding_available?/1","type":"callback"},{"doc":"Returns the etag for the current entity.This value will be used to respond to caching headers like If-None-Match.","ref":"Liberator.Resource.html#c:etag/1","title":"Liberator.Resource.etag/1","type":"callback"},{"doc":"Check if the etag for the current resource matches the value in the If-Match header.By default, checks the header against the value returned by etag/1.","ref":"Liberator.Resource.html#c:etag_matches_for_if_match?/1","title":"Liberator.Resource.etag_matches_for_if_match?/1","type":"callback"},{"doc":"Check if the etag of the current resource matches the If-Match-None header.By default, checks the header against the value returned by etag/1.","ref":"Liberator.Resource.html#c:etag_matches_for_if_none?/1","title":"Liberator.Resource.etag_matches_for_if_none?/1","type":"callback"},{"doc":"Check if the resource ever existed.Answering true here will lead you down the path that leads to responses like &quot;Moved Permanently&quot; and &quot;Gone&quot;, among othes.","ref":"Liberator.Resource.html#c:existed?/1","title":"Liberator.Resource.existed?/1","type":"callback"},{"doc":"Check if the requested entity exists.This is a great place to actually fetch the requested resource, then return it as a map so it can be merged into the :assigns map of the request.Returning false here will cause the plug to return a 404 Not Found response.","ref":"Liberator.Resource.html#c:exists?/1","title":"Liberator.Resource.exists?/1","type":"callback"},{"doc":"Returns content for a 202 Accepted response.","ref":"Liberator.Resource.html#c:handle_accepted/1","title":"Liberator.Resource.handle_accepted/1","type":"callback"},{"doc":"Returns content for a 409 Conflict response.","ref":"Liberator.Resource.html#c:handle_conflict/1","title":"Liberator.Resource.handle_conflict/1","type":"callback"},{"doc":"Returns content for a 201 Created response.","ref":"Liberator.Resource.html#c:handle_created/1","title":"Liberator.Resource.handle_created/1","type":"callback"},{"doc":"Returns a conn for a 500 Internal Server Error response.This handler is special among the handlers. Not only does it receive the conn, it also receives the error that was thrown, as well as the atom name of the failed step.","ref":"Liberator.Resource.html#c:handle_error/3","title":"Liberator.Resource.handle_error/3","type":"callback"},{"doc":"Returns content for a 403 Forbidden response.","ref":"Liberator.Resource.html#c:handle_forbidden/1","title":"Liberator.Resource.handle_forbidden/1","type":"callback"},{"doc":"Returns content for a 410 Gone response.","ref":"Liberator.Resource.html#c:handle_gone/1","title":"Liberator.Resource.handle_gone/1","type":"callback"},{"doc":"Returns content for a 400 Malformed response.","ref":"Liberator.Resource.html#c:handle_malformed/1","title":"Liberator.Resource.handle_malformed/1","type":"callback"},{"doc":"Returns content for a 405 Method Not Allowed response.","ref":"Liberator.Resource.html#c:handle_method_not_allowed/1","title":"Liberator.Resource.handle_method_not_allowed/1","type":"callback"},{"doc":"Returns content for a 301 Moved Permanently response.","ref":"Liberator.Resource.html#c:handle_moved_permanently/1","title":"Liberator.Resource.handle_moved_permanently/1","type":"callback"},{"doc":"Returns content for a 307 Moved Permanently response.","ref":"Liberator.Resource.html#c:handle_moved_temporarily/1","title":"Liberator.Resource.handle_moved_temporarily/1","type":"callback"},{"doc":"Returns content for a 300 Multiple Representations response.","ref":"Liberator.Resource.html#c:handle_multiple_representations/1","title":"Liberator.Resource.handle_multiple_representations/1","type":"callback"},{"doc":"Returns content for a 204 No Content response.","ref":"Liberator.Resource.html#c:handle_no_content/1","title":"Liberator.Resource.handle_no_content/1","type":"callback"},{"doc":"Returns content for a 406 Not Acceptable response.","ref":"Liberator.Resource.html#c:handle_not_acceptable/1","title":"Liberator.Resource.handle_not_acceptable/1","type":"callback"},{"doc":"Returns content for a 404 Not Found response.","ref":"Liberator.Resource.html#c:handle_not_found/1","title":"Liberator.Resource.handle_not_found/1","type":"callback"},{"doc":"Returns content for a 501 Not Implemented response.","ref":"Liberator.Resource.html#c:handle_not_implemented/1","title":"Liberator.Resource.handle_not_implemented/1","type":"callback"},{"doc":"Returns content for a 304 Not Modified response.","ref":"Liberator.Resource.html#c:handle_not_modified/1","title":"Liberator.Resource.handle_not_modified/1","type":"callback"},{"doc":"Returns content for a 200 OK response.","ref":"Liberator.Resource.html#c:handle_ok/1","title":"Liberator.Resource.handle_ok/1","type":"callback"},{"doc":"Returns content for a 200 OK response to an OPTIONS request.","ref":"Liberator.Resource.html#c:handle_options/1","title":"Liberator.Resource.handle_options/1","type":"callback"},{"doc":"Returns content for a 402 Payment Required response.Please note that the 402 status code is experimental, and is &quot;reserved for future use.&quot;","ref":"Liberator.Resource.html#c:handle_payment_required/1","title":"Liberator.Resource.handle_payment_required/1","type":"callback"},{"doc":"Returns content for a 412 Precondition Failed response.","ref":"Liberator.Resource.html#c:handle_precondition_failed/1","title":"Liberator.Resource.handle_precondition_failed/1","type":"callback"},{"doc":"Returns content for a 413 Entity Too Large response.","ref":"Liberator.Resource.html#c:handle_request_entity_too_large/1","title":"Liberator.Resource.handle_request_entity_too_large/1","type":"callback"},{"doc":"Returns content for a 303 See Other response.","ref":"Liberator.Resource.html#c:handle_see_other/1","title":"Liberator.Resource.handle_see_other/1","type":"callback"},{"doc":"Returns content for a 503 Service Unavailable response.","ref":"Liberator.Resource.html#c:handle_service_unavailable/1","title":"Liberator.Resource.handle_service_unavailable/1","type":"callback"},{"doc":"Returns content for a 429 Too Many Requests response.For more information on this response type, see RFC 6585, section 4.","ref":"Liberator.Resource.html#c:handle_too_many_requests/1","title":"Liberator.Resource.handle_too_many_requests/1","type":"callback"},{"doc":"Returns content for a 401 Unauthorized response.","ref":"Liberator.Resource.html#c:handle_unauthorized/1","title":"Liberator.Resource.handle_unauthorized/1","type":"callback"},{"doc":"Returns content for a 451 Unavailable for Legal Reasons response.For more information on this response type, see RFC 7725.","ref":"Liberator.Resource.html#c:handle_unavailable_for_legal_reasons/1","title":"Liberator.Resource.handle_unavailable_for_legal_reasons/1","type":"callback"},{"doc":"Returns content for a 501 Unknown Method response.","ref":"Liberator.Resource.html#c:handle_unknown_method/1","title":"Liberator.Resource.handle_unknown_method/1","type":"callback"},{"doc":"Returns content for a 422 Unprocesable Entity response.","ref":"Liberator.Resource.html#c:handle_unprocessable_entity/1","title":"Liberator.Resource.handle_unprocessable_entity/1","type":"callback"},{"doc":"Returns content for a 415 Unsuppported Media Type response.","ref":"Liberator.Resource.html#c:handle_unsupported_media_type/1","title":"Liberator.Resource.handle_unsupported_media_type/1","type":"callback"},{"doc":"Returns content for a 414 URI Too Long response.","ref":"Liberator.Resource.html#c:handle_uri_too_long/1","title":"Liberator.Resource.handle_uri_too_long/1","type":"callback"},{"doc":"Check if the If-Match header exists.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_match_exists?/1","title":"Liberator.Resource.if_match_exists?/1","type":"callback"},{"doc":"Check if the If-Match header is *.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_match_star?/1","title":"Liberator.Resource.if_match_star?/1","type":"callback"},{"doc":"Check if the If-Match * header exists for a resource that does not exist.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_match_star_exists_for_missing?/1","title":"Liberator.Resource.if_match_star_exists_for_missing?/1","type":"callback"},{"doc":"Check if the If-Modified-Since header exists.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_modified_since_exists?/1","title":"Liberator.Resource.if_modified_since_exists?/1","type":"callback"},{"doc":"Check if the If-Modified-Since header is a valid HTTP date.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_modified_since_valid_date?/1","title":"Liberator.Resource.if_modified_since_valid_date?/1","type":"callback"},{"doc":"Check if the request method to handle failed if-none-match.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_none_match?/1","title":"Liberator.Resource.if_none_match?/1","type":"callback"},{"doc":"Check if the If-None-Match header exists.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_none_match_exists?/1","title":"Liberator.Resource.if_none_match_exists?/1","type":"callback"},{"doc":"Check if the If-None-Match header is *.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_none_match_star?/1","title":"Liberator.Resource.if_none_match_star?/1","type":"callback"},{"doc":"Check if the If-Unmodified-Since header exists.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_unmodified_since_exists?/1","title":"Liberator.Resource.if_unmodified_since_exists?/1","type":"callback"},{"doc":"Check if the If-Unmodified-Since header is a valid HTTP date.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:if_unmodified_since_valid_date?/1","title":"Liberator.Resource.if_unmodified_since_valid_date?/1","type":"callback"},{"doc":"Callback implementation for Plug.init/1.","ref":"Liberator.Resource.html#init/1","title":"Liberator.Resource.init/1","type":"function"},{"doc":"A hook invoked at the beginning of the decision tree to set up anything you may need.You can return a map here and it will be merged with the given conn's :assigns map.","ref":"Liberator.Resource.html#c:initialize/1","title":"Liberator.Resource.initialize/1","type":"callback"},{"doc":"Check if the request method is Options.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:is_options?/1","title":"Liberator.Resource.is_options?/1","type":"callback"},{"doc":"Check if the Content-Type of the body is known.By default, always returns true.","ref":"Liberator.Resource.html#c:known_content_type?/1","title":"Liberator.Resource.known_content_type?/1","type":"callback"},{"doc":"Check of the HTTP method in the request is one we know about.This is different from allowed_methods/1 in that this function checks to see if the given HTTP method is an HTTP method at all. You probably want to override allowed_methods/1 and not this one, unless you're extending HTTP with more verbs.If this function returns false, then the plug will return a 501 Unknown Method response.By default, allows the methods returned by known_methods/1.","ref":"Liberator.Resource.html#c:known_method?/1","title":"Liberator.Resource.known_method?/1","type":"callback"},{"doc":"Returns a list of HTTP methods that exist.Note that this is to filter bad HTTP requests, not to filter requests that your endpoint does not serve. You probably want to implement allowed_methods/1 instead.The methods returned by this function should be upper-case strings, like &quot;GET&quot;, &quot;POST&quot;, etc.","ref":"Liberator.Resource.html#c:known_methods/1","title":"Liberator.Resource.known_methods/1","type":"callback"},{"doc":"Check if the requested language is available.By default, uses the values returned by available_languages/1, and returns a map with the key :language set to the negotiated language, which will be merged into conn.assigns.","ref":"Liberator.Resource.html#c:language_available?/1","title":"Liberator.Resource.language_available?/1","type":"callback"},{"doc":"Returns the last modified date of your resource.This value will be used to respond to caching headers like If-Modified-Since.","ref":"Liberator.Resource.html#c:last_modified/1","title":"Liberator.Resource.last_modified/1","type":"callback"},{"doc":"Check the request for general adherence to some form.If this function returns true, then the plug will return a 400 Malformed response.If you're checking the body of a request against some schema, you should override processable?/1 instead.By default, always returns false.","ref":"Liberator.Resource.html#c:malformed?/1","title":"Liberator.Resource.malformed?/1","type":"callback"},{"doc":"Configures the maximum length that a request body can be.","ref":"Liberator.Resource.html#c:maximum_entity_length/1","title":"Liberator.Resource.maximum_entity_length/1","type":"callback"},{"doc":"Check if the request media type is available.By default, uses the values returned by available_media_types/1.","ref":"Liberator.Resource.html#c:media_type_available?/1","title":"Liberator.Resource.media_type_available?/1","type":"callback"},{"doc":"Check if the server supports the request's HTTP method.Override allowed_methods/1 instead of this function to let this plug perform the check for you.By default, allows the methods returned by allowed_methods/1.","ref":"Liberator.Resource.html#c:method_allowed?/1","title":"Liberator.Resource.method_allowed?/1","type":"callback"},{"doc":"Check if the request method is DELETE.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:method_delete?/1","title":"Liberator.Resource.method_delete?/1","type":"callback"},{"doc":"Check if the request method is PATCH.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:method_patch?/1","title":"Liberator.Resource.method_patch?/1","type":"callback"},{"doc":"Check if the request method is POST.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:method_post?/1","title":"Liberator.Resource.method_post?/1","type":"callback"},{"doc":"Check if the request method is PUT.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:method_put?/1","title":"Liberator.Resource.method_put?/1","type":"callback"},{"doc":"Checks if the resource was modified since the date given in the If-Modified-Since header.By default, checks the header against the value returned by last_modified/1.","ref":"Liberator.Resource.html#c:modified_since?/1","title":"Liberator.Resource.modified_since?/1","type":"callback"},{"doc":"Check if the resource was moved permanently.By default, always returns false.","ref":"Liberator.Resource.html#c:moved_permanently?/1","title":"Liberator.Resource.moved_permanently?/1","type":"callback"},{"doc":"Check if the resource was moved temporarily.By default, always returns false.","ref":"Liberator.Resource.html#c:moved_temporarily?/1","title":"Liberator.Resource.moved_temporarily?/1","type":"callback"},{"doc":"Check if there are multiple representations of the resource.","ref":"Liberator.Resource.html#c:multiple_representations?/1","title":"Liberator.Resource.multiple_representations?/1","type":"callback"},{"doc":"Was the resource created by this request?","ref":"Liberator.Resource.html#c:new?/1","title":"Liberator.Resource.new?/1","type":"callback"},{"doc":"Called for PATCH requests.","ref":"Liberator.Resource.html#c:patch!/1","title":"Liberator.Resource.patch!/1","type":"callback"},{"doc":"Check if the PATCH request was processed. Return false here if the request was put on some processing queue and the patch was not actually enacted yet. Returning false here would return a 202 Accepted instead of some other response.By default, always returns true.","ref":"Liberator.Resource.html#c:patch_enacted?/1","title":"Liberator.Resource.patch_enacted?/1","type":"callback"},{"doc":"Check to see if payment is required for this resource.If this function returns true, then the plug will return a 402 Payment Required response. Please note that the 402 status code is experimental, and is &quot;reserved for future use.&quot;By default, always returns false.","ref":"Liberator.Resource.html#c:payment_required?/1","title":"Liberator.Resource.payment_required?/1","type":"callback"},{"doc":"Called for POST requests.","ref":"Liberator.Resource.html#c:post!/1","title":"Liberator.Resource.post!/1","type":"callback"},{"doc":"Check if the POST request was processed. Return false here if the request was put on some processing queue and the post was not actually enacted yet. Returning false here would return a 202 Accepted instead of some other response.By default, always returns true.","ref":"Liberator.Resource.html#c:post_enacted?/1","title":"Liberator.Resource.post_enacted?/1","type":"callback"},{"doc":"Decide if the response should redirect after a POST.By default, always returns false.","ref":"Liberator.Resource.html#c:post_redirect?/1","title":"Liberator.Resource.post_redirect?/1","type":"callback"},{"doc":"Check if the request method is POST for a resource that already exists.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:post_to_existing?/1","title":"Liberator.Resource.post_to_existing?/1","type":"callback"},{"doc":"Check if the request method is POST for resources that do not exist anymore.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:post_to_gone?/1","title":"Liberator.Resource.post_to_gone?/1","type":"callback"},{"doc":"Check if the request method is POST to a resource that doesn't exist.Used internally; it is not advised to override this function.","ref":"Liberator.Resource.html#c:post_to_missing?/1","title":"Liberator.Resource.post_to_missing?/1","type":"callback"},{"doc":"Check if the body of the request can be processed.This is a good place to parse a JSON body if that's what you're doing. Returning false here would cause the plug to return a 422 Unprocessable response.","ref":"Liberator.Resource.html#c:processable?/1","title":"Liberator.Resource.processable?/1","type":"callback"},{"doc":"Called for PUT requests.","ref":"Liberator.Resource.html#c:put!/1","title":"Liberator.Resource.put!/1","type":"callback"},{"doc":"Check if the PUT request was processed. Return false here if the request was put on some processing queue and the put was not actually enacted yet. Returning false here would return a 202 Accepted instead of some other response.By default, always returns true.","ref":"Liberator.Resource.html#c:put_enacted?/1","title":"Liberator.Resource.put_enacted?/1","type":"callback"},{"doc":"Decide if a PUT request should be made to a different URL.By default, always returns false.","ref":"Liberator.Resource.html#c:put_to_different_url?/1","title":"Liberator.Resource.put_to_different_url?/1","type":"callback"},{"doc":"Check if the request method is a PUT for a resource that already exists.","ref":"Liberator.Resource.html#c:put_to_existing?/1","title":"Liberator.Resource.put_to_existing?/1","type":"callback"},{"doc":"Should the response contain a representation of the resource?","ref":"Liberator.Resource.html#c:respond_with_entity?/1","title":"Liberator.Resource.respond_with_entity?/1","type":"callback"},{"doc":"Check if your service is available.This is the first function called in the entire pipeline, and lets you check to make sure everything works before going deeper. If this function returns false, then the plug will return a 503 Service Not Available response.If this function returns a map containing a value called :retry_after, Liberator will put this value into a retry-after header, Some crawlers and spiders honor this value, so they will not bother you while you're down, and will continue to index your site afterward. See MDN's docs on the retry-after header for more information.By default, always returns true.","ref":"Liberator.Resource.html#c:service_available?/1","title":"Liberator.Resource.service_available?/1","type":"callback"},{"doc":"Check to see if the client has performed too many requests. Used as part of a rate limiting scheme.If you return a map containing a :retry_after key, then the response's retry-after header will be automatically set. The value of this key can be either an Elixir DateTime object, a String HTTP date, or an integer of seconds. All of these values tell the client when they can attempt their request again. Note that if you provide a String for this value, it should be formatted as an HTTP date.If you do return map with the key :retry_after set, and its value is not a DateTime, integer, or valid String, then Liberator will raise an exception.By default, always returns false.","ref":"Liberator.Resource.html#c:too_many_requests?/1","title":"Liberator.Resource.too_many_requests?/1","type":"callback"},{"doc":"Check if the resource is no longer available, for legal reasons.If this function returns true, then the plug will return a 451 Unavailable for Legal Reasons response.By default, always returns false.","ref":"Liberator.Resource.html#c:unavailable_for_legal_reasons?/1","title":"Liberator.Resource.unavailable_for_legal_reasons?/1","type":"callback"},{"doc":"Checks if the resource was not modified since the date given in the If-Unmodified-Since header.By default, checks the header against the value returned by last_modified/1.","ref":"Liberator.Resource.html#c:unmodified_since?/1","title":"Liberator.Resource.unmodified_since?/1","type":"callback"},{"doc":"Checks the length of the URI.If this function returns true, then the plug will return a 414 URI Too Long response.By default, always returns false.","ref":"Liberator.Resource.html#c:uri_too_long?/1","title":"Liberator.Resource.uri_too_long?/1","type":"callback"},{"doc":"Check if the Content-Type of the body is valid.By default, always returns true.","ref":"Liberator.Resource.html#c:valid_content_header?/1","title":"Liberator.Resource.valid_content_header?/1","type":"callback"},{"doc":"Check if the length of the body is valid.By default, always returns true.","ref":"Liberator.Resource.html#c:valid_entity_length?/1","title":"Liberator.Resource.valid_entity_length?/1","type":"callback"},{"doc":"Check the request for general adherence to some form.If this function returns false, then the plug will return a 400 Malformed response.This is a good place to parse the request body. If, for example, the body is not valid JSON, this is the function that should return false. If you're checking the body of a request against some schema, like checking if your changeset is valid, you should override processable?/1 instead.By default, always returns true.","ref":"Liberator.Resource.html#c:well_formed?/1","title":"Liberator.Resource.well_formed?/1","type":"callback"},{"doc":"Generates source text for a decision tree chart for a Liberator resource. The chart is compatible with the Graphviz graph visualization software.mix liberator.chartBy default, this function will print the default decision tree to standard output. This task can also take a module argument for any module that uses Liberator.Resource, in which case the decision tree for the given module will be printed.mix liberator.chart MyApp.MyResourceYou can also provide the --output or -o option to print the chart source to a file.mix liberator.chart -o myresource.dot MyApp.MyResourceGenerating a chart with the returned source codeUnfortunately, there's not a Graphviz binding for Elixir. If you want to create an actual image of your chart, you will have to install Graphviz, or use one of its language bindings for another language.Once you have installed Graphviz, you can run a command like the following to generate an imagedot myresource.dot -Tsvg -o myresource.svg","ref":"Mix.Tasks.Liberator.Chart.html","title":"Mix.Tasks.Liberator.Chart","type":"task"},{"doc":"Callback implementation for Mix.Task.run/1.","ref":"Mix.Tasks.Liberator.Chart.html#run/1","title":"Mix.Tasks.Liberator.Chart.run/1","type":"function"},{"doc":"An implementation of the deflate compression used in HTTP.","ref":"Liberator.Encoding.Deflate.html","title":"Liberator.Encoding.Deflate","type":"module"},{"doc":"An implementation of the gzip compression used in HTTP.","ref":"Liberator.Encoding.Gzip.html","title":"Liberator.Encoding.Gzip","type":"module"},{"doc":"An implementation of the identity compression used in HTTP. That is to say, a no-op function that returns its argument.","ref":"Liberator.Encoding.Identity.html","title":"Liberator.Encoding.Identity","type":"module"},{"doc":"An implementation of a text/plain encoder for HTTP.","ref":"Liberator.MediaType.TextPlain.html","title":"Liberator.MediaType.TextPlain","type":"module"},{"doc":"Decision tracing functions.","ref":"Liberator.Trace.html","title":"Liberator.Trace","type":"module"},{"doc":"Get the log of all decisions made on the given conn.The trace is a list of maps, each map corresponding to one call to a decision function. Each map has the following keys::step: the name of the function that was executed, or the atoms :start or :stop:result: the value the function returned:timestamp: the time the function was called:duration: how long the call took, in native time units","ref":"Liberator.Trace.html#get_trace/1","title":"Liberator.Trace.get_trace/1","type":"function"},{"doc":"Get a list of tuples for the x-liberator-trace header, based on the given trace.","ref":"Liberator.Trace.html#headers/1","title":"Liberator.Trace.headers/1","type":"function"},{"doc":"Log a message containing the given trace, along with its request path and optional request ID.","ref":"Liberator.Trace.html#log/3","title":"Liberator.Trace.log/3","type":"function"},{"doc":"A provided compression codec returned an invalid value.The Liberator.Encoding.encode!/1 function must return a binary.","ref":"Liberator.CompressionCodecInvalidResult.html","title":"Liberator.CompressionCodecInvalidResult","type":"exception"},{"doc":"Liberator was unable to find a compression codec for the given encoding.","ref":"Liberator.CompressionCodecNotFound.html","title":"Liberator.CompressionCodecNotFound","type":"exception"},{"doc":"Liberator was unable to process the given :last_modified value into an HTTP date.","ref":"Liberator.InvalidLastModifiedValue.html","title":"Liberator.InvalidLastModifiedValue","type":"exception"},{"doc":"Liberator was unable to process the given :retry_after value into an HTTP date.","ref":"Liberator.InvalidRetryAfterValue.html","title":"Liberator.InvalidRetryAfterValue","type":"exception"},{"doc":"A provided media type codec returned an invalid value.The Liberator.MediaType.encode!/1 function must return a binary.","ref":"Liberator.MediaTypeCodecInvalidResult.html","title":"Liberator.MediaTypeCodecInvalidResult","type":"exception"},{"doc":"Liberator was unable to find a media type codec for the given encoding.","ref":"Liberator.MediaTypeCodecNotFound.html","title":"Liberator.MediaTypeCodecNotFound","type":"exception"},{"doc":"Liberator's decision tree evaluator encountered a step that it doesn't know what to do with.","ref":"Liberator.UnknownStep.html","title":"Liberator.UnknownStep","type":"exception"},{"doc":"Liberator An Elixir library for building RESTful applications.Liberator is a port of the Liberator Clojure library that allows you to define a controller that adheres to the HTTP spec by providing just a few pieces of information. It implements a decision graph of simple boolean questions that lead your app to the correct HTTP status codes.While Phoenix and Plug make routing easy, they don't do anything with content negotiation, or cache management, or existence checks, or anything like that, beyond calling the right controller function based on the HTTP method. There are a lot of decisions to make before returning the right HTTP status code, but Phoenix doesn't give you any additional power to do so. Liberator does.","ref":"readme.html","title":"Liberator","type":"extras"},{"doc":"This package is available in Hex. Install it by adding liberator to your list of dependencies in mix.exs:def deps do [ {:liberator, &quot;~&gt; 1.4.0&quot;} ] endDocumentation can be generated with ExDoc and can be found online at https://hexdocs.pm/liberator.","ref":"readme.html#install","title":"Liberator - Install","type":"extras"},{"doc":"For a basic GET endpoint, you can define an entire module in five lines of code. Technically you don't even need to implement these two, since sensible defaults are provided.defmodule MyFirstResource do use Liberator.Resource def available_media_types(_), do: [&quot;text/plain&quot;] def handle_ok(_), do: &quot;Hello world!&quot; endIt doesn't look like much, but behind the scenes, Liberator navigated a decision graph of content negotation, cache management, and existence checks before returning 200 OK. Liberator finds the best media type you support, and automatically encodes your return value. JSON is supported out of the box, and any additional types can be provided in a line of the config.# in config.exs config :liberator, media_types: %{ &quot;application/json&quot; =&gt; Jason, &quot;application/xml&quot; =&gt; MyXmlCodec } # in your main body of code defmodule MyJsonOrXmlResource do use Liberator.Resource def available_media_types(_), do: [&quot;application/json&quot;, &quot;application/xml&quot;] def handle_ok(_), do: %{message: &quot;hi!&quot;} endA Liberator Resource implements the Plug spec, so you can forward requests to it in frameworks like Phoenix:scope &quot;/api&quot;, MyApp do pipe_through [:api] forward &quot;/resources&quot;, MyFirstResource endYour results from questions are aggregated into the :assigns map on the conn, so you don't have to access data more than once.defmodule MaybeExistingResource do use Liberator.Resource def exists?(conn) do case MyApp.Repo.get(MyApp.Post, conn.params[&quot;id&quot;]) do nil -&gt; false post -&gt; %{post: post} end end def handle_ok(conn), do: conn.assigns[:post] endSee more in the Getting Started guide, and in the documentation for Liberator.Resource.","ref":"readme.html#usage","title":"Liberator - Usage","type":"extras"},{"doc":"This project was developed by Rosa Richter. You can get in touch with her on Keybase.io.","ref":"readme.html#maintainer","title":"Liberator - Maintainer","type":"extras"},{"doc":"Thanks to the maintainers of the original Clojure liberator project, Philipp Meier and Malcolm Sparks, for creating such a handy tool. Their great documentation was an immense help in porting it to Elixir. And thanks to the maintainers of Erlang's webmachine for inspiring them!","ref":"readme.html#thanks","title":"Liberator - Thanks","type":"extras"},{"doc":"Questions and pull requests are more than welcome. I follow Elixir's tenet of bad documentation being a bug, so if anything is unclear, please file an issue! Ideally, my answer to your question will be in an update to the docs.Please see CONTRIBUTING.md for all the details you could ever want about helping me with this project.Note that this project is released with a Contributor Code of Conduct. By participating in this project you agree to abide by its terms.","ref":"readme.html#contributing","title":"Liberator - Contributing","type":"extras"},{"doc":"MIT LicenseCopyright 2020 Rosa RichterPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"readme.html#license","title":"Liberator - License","type":"extras"},{"doc":"ChangelogAll notable changes to this project will be documented in this file.The format is based on Keep a Changelog, and this project adheres to Semantic Versioning.","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-4-0-2021-03-23","title":"Changelog - 1.4.0 - 2021-03-23","type":"extras"},{"doc":"New CallbacksThe handle_error/3 handler has been added. This is a special handler that lets you manipulate the conn after an error is raised, so you can choose your status code and body yourself. (docs) (#33)The maximum_entity_length/1 function. This is used by the valid_entity_length?/1 callback. If the size of the request body is above this size, Liberator will return a status of 413 Request Entity Too Large. (docs)Added the well_formed?/1 function. This function checks the request for general adherence to some form, like if it is valid JSON. It replaces malformed?/1 so that you can return some data after you check for form. (docs)Added the body_exists?/1 function. This is an internal function that, uh, checks to see if the body exists. If it does, it'll grab it, and call valid_entity_length?/1 and the new well_formed?/1 function. So now you can parse the body if it's there, without worrying about conditional logic if it's not there. (docsAdditions to callbacksYou can now return the given conn in decision functions and actions to serve as an affirmative response, as an alternative to returning a plain map, or true. Now you can modify the conn as you see fit.You can now return :ok or :error tuples from decisions, actions, and handlers. Returning {:error, term} will invoke the new handle_error/3 function.Docs for callbacksAdded internationalizationInternationalization is now supported via Gettext! Liberator already finds the best language for each request, now it also sets the locale using Gettext.put_locale/1. Just call gettext directly in your Resources. (docs) (#8, #10)Debugging and tracing upgradesTracing individual step duration is now available. Access the :duration key for each decision in the tracing list to get that decision's duration, in native time units.You can now generate a Graphviz file for either the default decision tree or your own resource's decision tree using the mix liberator.chart mix task.Telemetry is now sent upon each request. Three events [:liberator, :request, :start], [:liberator, :request, :stop], and [:liberator, :request, :exception] are sent. See the docs for Liberator.Resource for more information.Docs for debugging and tracing","ref":"changelog.html#added","title":"Changelog - Added","type":"extras"},{"doc":"The Vary header is now served by default, with a value of Accept-Encoding and Accept-Language.Now serves the location header if you've returned a map with the :location key, or assigned it on the conn.Some decision functions were rearranged.Raised errors are now wrapped in custom exceptions. They're the same errors with the same messages, just with different wrapper types.Entries in the trace list have changed from tuples to maps in order to support more tracing information. The step is now a map member named :step, and the value is :value","ref":"changelog.html#changed","title":"Changelog - Changed","type":"extras"},{"doc":"The malformed?/1 is now deprecated, use well_formed?/1 instead. This lets that decision function return data, and it's the ideal place for parsing the body. (#15)","ref":"changelog.html#deprecated","title":"Changelog - Deprecated","type":"extras"},{"doc":"The etag header is now included in the response, if you have overridden it. (#17)Now serves the last-modified header based on the return value from the last_modified/1 callback. (#18)Non-printable-String return values from handlers will now be passed through inspect/1 when the content type is text/plain. Printable strings will be passed through without hassle. (#7)All responses now include an allow header, fixing the cases where one was required but not provided, like in an OPTIONS request, and when returning a 405 Method Not Allowed. (#9, #12)","ref":"changelog.html#fixed","title":"Changelog - Fixed","type":"extras"},{"doc":"AddedThe :log argument to the :trace option. Add a trace: :log option to your module's use statement, and every request will be logged with a trace. The log message even includes a request ID from Plug.RequestId, if you have one. (#4)The :decision_tree_overrides argument to Liberator.Resource. This is an advanced feature that lets users override parts of the decision tree that Liberator evaluates. Want to change the ordering of some decisions, or add new ones? This is the way! (#5)The :handler_status_overrides argument to Liberator.Resource. This is an advanced feature that lets users override status codes associated with handlers. It's mostly useful for those wanting to implement their own new statuses that Liberator doesn't support.The :action_followup_overrides argument to Liberator.Resource. This is an advanced feature that lets users override the functions called immediately after actions. It's mostly useful for those wanting to implement their own new actions that Liberator doesn't support.","ref":"changelog.html#1-3-0-2020-10-13","title":"Changelog - 1.3.0 - 2020-10-13","type":"extras"},{"doc":"AddedThe 402 Payment Required status, along with related callbacks payment_required? and handle_payment_required/1.The 451 Unavailable for Legal Reasons status, along with related callbacks unavailable_for_legal_reasons?/1 and handle_unavailable_for_legal_reasons/1.The 429 Too Many Requests status, along with related callbacks too_many_requests?/1 and handle_too_many_requests/1. If you return a map containing a :retry_after value, Liberator will use that to set a retry-after header.You can also return a :retry_after value from any other decision function, like service_available?/1, or moved_permanently?/1, for the same effect. See MDN's docs on the retry-after header for more information on why you'd want to do this.FixedDates in headers are now parsed properly. (#1)","ref":"changelog.html#1-2-0-2020-10-12","title":"Changelog - 1.2.0 - 2020-10-12","type":"extras"},{"doc":"AddedThis changelog!The :trace option: Add trace: :headers to your use Liberator.Resource statement to get an x-liberator-trace header added to responses, and see the result of all decisions.Compression options: deflate, gzip, and identity.","ref":"changelog.html#1-1-0-2020-10-04","title":"Changelog - 1.1.0 - 2020-10-04","type":"extras"},{"doc":"Codecs are now configurable. Set the :media_types and :encodings map in Liberator's config to add your own codecs.RemovedLiberator.Resource no longer calls use Timex, so your context is less polluted.","ref":"changelog.html#changed","title":"Changelog - Changed","type":"extras"},{"doc":"Better wildcard handling during content negotiationContent negotiation actually obeys q-values for priority","ref":"changelog.html#fixed","title":"Changelog - Fixed","type":"extras"},{"doc":"AddedBasic decision tree navigation","ref":"changelog.html#1-0-0-2020-10-02","title":"Changelog - 1.0.0 - 2020-10-02","type":"extras"},{"doc":"ContributingThank you for considering contributing! I write libraries in my free time and contributions from others help me make great tools.Following these guidelines helps to communicate that you respect my time, as the developer managing and developing this open source project. In return, I should reciprocate that respect in addressing your issue, assessing changes, and helping you finalize your pull requests.Questions and pull requests are more than welcome. I follow Elixir's tenet of bad documentation being a bug, so if anything is unclear, please file an issue! Ideally, my answer to your question will be in an update to the docs.Please note that this project is released with a Contributor Code of Conduct. By participating in this project you agree to abide by its terms.Working on your first Pull Request? You can learn how from this free series How to Contribute to an Open Source Project on GitHub.","ref":"contributing.html","title":"Contributing","type":"extras"},{"doc":"Follow the Code of Conduct.Keep your commits clear and your pull requests small. This isn't a big library.Run mix format on your files before committing them. I like clean diffs.Run mix credo and resolve anything of yours that comes up.Delegate as much as possible to the user, or let them override everything.Provide sensible defaults. If you can use Liberator.Resource in an empty module in an empty Phoenix project, and make a request that returns 200 OK, you're doing great.Don't pollute the user's context.","ref":"contributing.html#ground-rules","title":"Contributing - Ground Rules","type":"extras"},{"doc":"Ask questions! I like writing good documentation, and questions make that work more meaningful. Use the issue tracker for questions. There's a questions tag on the issue tracker for that reason.Tests are always welcome! Liberator is a project with a lot of conditional logic, with a lot of resulting complexity.Searching issues or pull requests tagged &quot;help wanted&quot; or &quot;good first issue&quot; are great places to get started.","ref":"contributing.html#your-first-contribution","title":"Contributing - Your First Contribution","type":"extras"},{"doc":"GitHub makes it really easy to submit pull requests. Just:Create your own fork of the codeDo the changes in your forkSubmit a pull requestI don't require a CLA or anything like that.","ref":"contributing.html#getting-started","title":"Contributing - Getting Started","type":"extras"},{"doc":"If you find a security vulnerability, do NOT open an issue. Contact Rosa on keybase.io instead!In order to determine whether you are dealing with a security issue, ask yourself these two questions:Can I access something that's not mine, or something I shouldn't have access to?Can I disable something for other people?If the answer to either of those two questions are &quot;yes&quot;, then you're probably dealing with a security issue. Note that even if you answer &quot;no&quot; to both questions, you may still be dealing with a security issue, so if you're unsure, message me directly. For more information on security, see SECURITY.md.Please open an issue and follow the directions in the issue template.","ref":"contributing.html#how-report-a-bug","title":"Contributing - How report a bug","type":"extras"},{"doc":"The philosopy of this project is to create a small, sharp tool that takes the burden of the HTTP spec off of the shoulders of developers. Feature requests and enhancements should stick to this philosophy. Help me implement the HTTP spec more closely, or help me do anything that lets developers do more with their valuable time.Feature requests give meaning to my work. Open an issue that decribes the feature you'd like to see, why you need it, and how you'd like it to work.","ref":"contributing.html#how-to-suggest-a-feature-or-enhancement","title":"Contributing - How to suggest a feature or enhancement","type":"extras"},{"doc":"As the primary developer, I will be the one reviewing all pull requests. I check GitHub almost every day, so you should be able to hear back from me quickly. However, I only do this in my free time, so please allow me flexibility.","ref":"contributing.html#code-review-process","title":"Contributing - Code review process","type":"extras"},{"doc":"Chat with me on https://gitter.im/liberator-elixir/community. This is a small project, but I care about the people that care about my work. I'm also on keybase if you need to message me directly.If you like this contribution guide, please give a star to Nadia Eghbal's contributing-template project on GitHub. It was a great help.","ref":"contributing.html#community","title":"Contributing - Community","type":"extras"},{"doc":"Security Policy","ref":"security.html","title":"Security Policy","type":"extras"},{"doc":"All versions of Liberator are currently supported with security updates.VersionSupported1.3.x:white_check_mark:1.2.x:white_check_mark:1.1.x:white_check_mark:1.0.x:white_check_mark:","ref":"security.html#supported-versions","title":"Security Policy - Supported Versions","type":"extras"},{"doc":"If you find a security vulnerability, do not open an issue. Directly contact Rosa on keybase.io instead. Here is the signed proof of ownership that proves both the GitHub and Keybase accounts are owned by the same person.You can also send an encrypted email to the primary UID of the PGP key 60BA2A8BFD583641. The signatures on Keybase were made with that key.Security fixes are my top priority, and I'll do my best to get it fixed within the day. If not, expect daily personal updates until it's fixed","ref":"security.html#reporting-a-vulnerability","title":"Security Policy - Reporting a Vulnerability","type":"extras"},{"doc":"Contributor Covenant Code of Conduct","ref":"code_of_conduct.html","title":"Contributor Covenant Code of Conduct","type":"extras"},{"doc":"We as members, contributors, and leaders pledge to make participation in our community a harassment-free experience for everyone, regardless of age, body size, visible or invisible disability, ethnicity, sex characteristics, gender identity and expression, level of experience, education, socio-economic status, nationality, personal appearance, race, religion, or sexual identity and orientation.We pledge to act and interact in ways that contribute to an open, welcoming, diverse, inclusive, and healthy community.","ref":"code_of_conduct.html#our-pledge","title":"Contributor Covenant Code of Conduct - Our Pledge","type":"extras"},{"doc":"Examples of behavior that contributes to a positive environment for our community include:Demonstrating empathy and kindness toward other peopleBeing respectful of differing opinions, viewpoints, and experiencesGiving and gracefully accepting constructive feedbackAccepting responsibility and apologizing to those affected by our mistakes, and learning from the experienceFocusing on what is best not just for us as individuals, but for the overall communityExamples of unacceptable behavior include:The use of sexualized language or imagery, and sexual attention or advances of any kindTrolling, insulting or derogatory comments, and personal or political attacksPublic or private harassmentPublishing others' private information, such as a physical or email address, without their explicit permissionOther conduct which could reasonably be considered inappropriate in a professional setting","ref":"code_of_conduct.html#our-standards","title":"Contributor Covenant Code of Conduct - Our Standards","type":"extras"},{"doc":"Community leaders are responsible for clarifying and enforcing our standards of acceptable behavior and will take appropriate and fair corrective action in response to any behavior that they deem inappropriate, threatening, offensive, or harmful.Community leaders have the right and responsibility to remove, edit, or reject comments, commits, code, wiki edits, issues, and other contributions that are not aligned to this Code of Conduct, and will communicate reasons for moderation decisions when appropriate.","ref":"code_of_conduct.html#enforcement-responsibilities","title":"Contributor Covenant Code of Conduct - Enforcement Responsibilities","type":"extras"},{"doc":"This Code of Conduct applies within all community spaces, and also applies when an individual is officially representing the community in public spaces. Examples of representing our community include using an official e-mail address, posting via an official social media account, or acting as an appointed representative at an online or offline event.","ref":"code_of_conduct.html#scope","title":"Contributor Covenant Code of Conduct - Scope","type":"extras"},{"doc":"Instances of abusive, harassing, or otherwise unacceptable behavior may be reported to the community leaders responsible for enforcement by sending a message to Rosa Richter on keybase.io. All complaints will be reviewed and investigated promptly and fairly.All community leaders are obligated to respect the privacy and security of the reporter of any incident.","ref":"code_of_conduct.html#enforcement","title":"Contributor Covenant Code of Conduct - Enforcement","type":"extras"},{"doc":"Community leaders will follow these Community Impact Guidelines in determining the consequences for any action they deem in violation of this Code of Conduct:1. CorrectionCommunity Impact: Use of inappropriate language or other behavior deemed unprofessional or unwelcome in the community.Consequence: A private, written warning from community leaders, providing clarity around the nature of the violation and an explanation of why the behavior was inappropriate. A public apology may be requested.2. WarningCommunity Impact: A violation through a single incident or series of actions.Consequence: A warning with consequences for continued behavior. No interaction with the people involved, including unsolicited interaction with those enforcing the Code of Conduct, for a specified period of time. This includes avoiding interactions in community spaces as well as external channels like social media. Violating these terms may lead to a temporary or permanent ban.3. Temporary BanCommunity Impact: A serious violation of community standards, including sustained inappropriate behavior.Consequence: A temporary ban from any sort of interaction or public communication with the community for a specified period of time. No public or private interaction with the people involved, including unsolicited interaction with those enforcing the Code of Conduct, is allowed during this period. Violating these terms may lead to a permanent ban.4. Permanent BanCommunity Impact: Demonstrating a pattern of violation of community standards, including sustained inappropriate behavior, harassment of an individual, or aggression toward or disparagement of classes of individuals.Consequence: A permanent ban from any sort of public interaction within the community.","ref":"code_of_conduct.html#enforcement-guidelines","title":"Contributor Covenant Code of Conduct - Enforcement Guidelines","type":"extras"},{"doc":"This Code of Conduct is adapted from the Contributor Covenant, version 2.0, available at https://www.contributor-covenant.org/version/2/0/code_of_conduct.html.Community Impact Guidelines were inspired by Mozilla's code of conduct enforcement ladder.For answers to common questions about this code of conduct, see the FAQ at https://www.contributor-covenant.org/faq. Translations are available at https://www.contributor-covenant.org/translations.","ref":"code_of_conduct.html#attribution","title":"Contributor Covenant Code of Conduct - Attribution","type":"extras"},{"doc":"Getting StartedLet's get a Liberator endpoint up and running as fast as possible.First, you'll probably want to fire up a Phoenix application. Check out Phoenix's Installation Guide and their Up and Running Guide and install it if you don't have it already, then create a new project with phx.new.Since Liberator was built to help create APIs, we will disable HTML generation. We'll keep Ecto, though.$ mix phx.new hello --no-html --no-webpackWe'll skip the details of setting up an Ecto repo, because it's not necessary to have one running to use Liberator. However, your data probably lives in a repo, so this example will contain some references to one.Add liberator to your list of dependencies in mix.exsdef deps do [ {:liberator, &quot;~&gt; 1.3.0&quot;} ] endThen we'll use mix phx.gen.json to create a basic schema, route, and context. It will create a controller as well, but we'll be replacing that with a Liberator Resource.$ mix phx.gen.json Blog Post posts title:string content:stringMigrate the database as the command feedback requests, if you've got your database handy. However, don't copy the statement they're asking you to put in lib/hello_web/router.ex. We're going to use our own. In lib/hello_web/router.ex, forward any /posts requests to the resource we're about to define.defmodule HelloWeb.Router do use HelloWeb, :router forward &quot;/posts&quot;, HelloWeb.PostsResource endNow we must actually create our first Liberator resource. Go ahead and delete the file at lib/controllers/posts_controller.ex that Phoenix generated for you, and create a file at lib/controllers/posts_resource.ex with the following content.defmodule HelloWeb.PostsResource do use Liberator.Resource endRun the project with mix phx.server and check out http://localhost:4000/posts. With Liberator's sensible defaults, it should return a status of 200 and content of OK.$ curl --location --request GET 'http://localhost:4000/posts/' HTTP/1.1 200 OK allow: GET, HEAD cache-control: max-age=0, private, must-revalidate content-encoding: identity content-length: 2 content-type: text/plain date: Thu, 15 Oct 2020 22:54:57 GMT server: Cowboy x-request-id: Fj5MYV-9JH1y19wAACQl OKLet's spice that up a little more and get some dynamic content in here.defmodule HelloWeb.PostsResource do use Liberator.Resource @impl true def handle_ok(_conn) do DateTime.utc_now() |&gt; DateTime.to_iso8601() end end$ curl --location --request GET 'http://localhost:4000/posts/' HTTP/1.1 200 OK allow: GET, HEAD cache-control: max-age=0, private, must-revalidate content-encoding: identity content-length: 27 content-type: text/plain date: Thu, 15 Oct 2020 22:53:42 GMT server: Cowboy x-request-id: Fj5MT91NPreSB7QAABqF 2020-10-15T22:53:30.668000ZThat's barely better than regular Phoenix controllers, so let's go further. Since we're trying to write an API, why don't we grab our Repo and try to get some data out in the world?defmodule HelloWeb.PostsResource do use Liberator.Resource @impl true def handle_ok(conn) do id = List.last(conn.path_info) post = Hello.Blog.get_post!(id) post.title &lt;&gt; &quot;\\n\\n&quot; &lt;&gt; post.content end endNow if you try to hit http://localhost:4000/posts/5, we get a Phoenix exception. Nothing is in the database, of course. We must check to see if the given ID exists in the database, and here's where Liberator's features start to show.Implement the Liberator.Resource.exists?/1 callback in your resource. Here we can check for the resource. In fact, let's just grab the entire thing and stick in back in the conn. The exists?/1 callback expects you to return a boolean, a Map, or the conn. If you return a Map, Liberator will merge that into conn.assigns.defmodule HelloWeb.PostsResource do use Liberator.Resource @impl true def exists?(conn) do id = List.last(conn.path_info) try do Hello.Blog.get_post!(id) rescue Ecto.NoResultsError -&gt; false ArgumentError -&gt; false else post -&gt; %{post: post} end end @impl true def handle_ok(conn) do post = conn.assigns[:post] post.title &lt;&gt; &quot; &quot; &lt;&gt; post.content end end$ curl --location --request GET 'http://localhost:4000/posts/1555' HTTP/1.1 404 Not Found allow: GET, HEAD cache-control: max-age=0, private, must-revalidate content-encoding: identity content-length: 9 content-type: text/plain date: Thu, 15 Oct 2020 22:52:25 GMT server: Cowboy x-request-id: Fj5MPb9fgMWho78AABkF Not FoundNow we get a nice &quot;Not Found&quot; response! So, last, we want to be able to POST something. That introduces us to the concept of actions, which are just another kind of overridable function. There are four actions in Liberator:delete!patch!post!put!As you could guess, these actions will be called based on the request's HTTP method. For now, we'll set up post!. Add Liberator.Resource.post!/1 to insert the params. We also want to make sure that we allow POST requests, so define Liberator.Resource.allowed_methods/1 too.We could parse the request body here if we wanted, but Plug provides the Plug.Parsers plug, and it's already set up in this beginner project for accepting JSON. The params are available in conn.params, so we can pass those into our context module directly, and let the changeset handle validation.defmodule HelloWeb.PostsResource do use Liberator.Resource @impl true def allowed_methods(_), do: [&quot;POST&quot;, &quot;GET&quot;] @impl true def exists?(conn) do id = List.last(conn.path_info) case Hello.Repo.get(Hello.Blog.Post, id) do nil -&gt; false post -&gt; %{post: post} end end @impl true def post!(conn) do {:ok, _post} = Hello.Blog.create_post(conn.params) end @impl true def handle_ok(conn) do post = conn.assigns[:post] post.title &lt;&gt; &quot; &quot; &lt;&gt; post.content end endThat's all! Let's POST to our new endpoint now.$ curl --location --request POST 'http://localhost:4000/posts' \\ &gt; --header 'Content-Type: application/json' \\ &gt; --data-raw '{ &gt; &quot;title&quot;: &quot;My first post!&quot;, &gt; &quot;content&quot;: &quot;This is so fun!&quot; &gt; }' HTTP/1.1 201 Created allow: POST, GET cache-control: max-age=0, private, must-revalidate content-encoding: identity content-length: 7 content-type: text/plain date: Thu, 15 Oct 2020 22:42:07 GMT server: Cowboy x-request-id: Fj5L-UYbiai2oD4AADaB CreatedFinally, point your browser at http://localhost:4000/posts/1.curl --location --request GET 'http://localhost:4000/posts/1' HTTP/1.1 200 OK allow: POST, GET cache-control: max-age=0, private, must-revalidate content-encoding: identity content-length: 30 content-type: text/plain date: Thu, 15 Oct 2020 22:47:30 GMT server: Cowboy x-request-id: Fj5LreF6rvXe6igAABoD My first post! This is so fun!You've done it! You've just built JSON endpoint with a fair amount of built-in smarts. There's so much more to customize, but now you've got a base to work from. Check out the documentation for Liberator.Resource to see all the stuff you can do!","ref":"getting_started.html","title":"Getting Started","type":"extras"},{"doc":"Decision TreeThere are a lot of decision to be made before Liberator sends a response. This is the current state of Liberator's decision tree, with handlers at the bottom.","ref":"decision_tree.html","title":"Decision Tree","type":"extras"},{"doc":"You can create Graphviz source code for your own decision tree using the liberator.chart mix task.mix liberator.chart -o mychart.dot MyApp.MyResourceYou will have to install Graphviz or use one of its many language bindings to generate the actual image. Unfortunately, there isn't a binding for Elixir or Erlang.","ref":"decision_tree.html#creating-your-own-chart","title":"Decision Tree - Creating your own chart","type":"extras"}]